@@ -3,6 +3,7 @@ import os
 import re
 import pysam
 from eta import ETA
+import ngsutils.support
 
 
 def bam_open(fname, mode='r', *args, **kwargs):
@@ -33,43 +34,93 @@ def bam_pileup_iter(bam, mask=1796, quiet=False, callback=None):
         eta.done()
 
 
-def bam_iter(bam, quiet=False, show_ref_pos=False, callback=None):
+def bam_iter(bam, quiet=False, show_ref_pos=False, ref=None, start=None, end=None, callback=None):
     '''
     >>> [x.qname for x in bam_iter(bam_open(os.path.join(os.path.dirname(__file__), 't', 'test.bam')), quiet=True)]
     ['A', 'B', 'E', 'C', 'D', 'F', 'Z']
     '''
 
-    
-    if not quiet and bam.filename:
-        eta = ETA(os.stat(bam.filename).st_size)
-    else:
-        eta = None
-
     if os.path.exists('%s.bai' % bam.filename):
         # This is an indexed file, so it is ref sorted...
         # Meaning that we should show chrom:pos, instead of read names
         show_ref_pos = True
 
-    for read in bam:
-        pos = bam.tell()
-        bgz_offset = pos >> 16
+    eta = None
 
-        if not quiet and eta:
-            if callback:
-                eta.print_status(bgz_offset, extra=callback(read))
-            elif (show_ref_pos):
-                if read.tid > -1:
-                    eta.print_status(bgz_offset, extra='%s:%s %s' % (bam.getrname(read.tid), read.pos, read.qname))
+    if not ref:
+        if not quiet and bam.filename:
+            eta = ETA(os.stat(bam.filename).st_size)
+
+        for read in bam:
+            pos = bam.tell()
+            bgz_offset = pos >> 16
+
+            if not quiet and eta:
+                if callback:
+                    eta.print_status(bgz_offset, extra=callback(read))
+                elif (show_ref_pos):
+                    if read.tid > -1:
+                        eta.print_status(bgz_offset, extra='%s:%s %s' % (bam.getrname(read.tid), read.pos, read.qname))
+                    else:
+                        eta.print_status(bgz_offset, extra='unmapped %s' % (read.qname))
                 else:
-                    eta.print_status(bgz_offset, extra='unmapped %s' % (read.qname))
-            else:
-                eta.print_status(bgz_offset, extra='%s' % read.qname)
-        yield read
+                    eta.print_status(bgz_offset, extra='%s' % read.qname)
+
+            yield read
+
+    else:
+        working_chrom = None
+        if ref in bam.references:
+            working_chrom = ref
+        elif ref[0:3] == 'chr':
+            # compensate for Ensembl vs UCSC ref naming
+            if ref[3:] in bam.references:
+                working_chrom = ref[3:]
+
+        if not working_chrom:
+            raise ValueError('Missing reference: %s' % ref)
+
+        tid = bam.gettid(working_chrom)
+
+        if not start:
+            start = 0
+        if not end:
+            end = bam.lengths[tid]
+
+        if not quiet and bam.filename:
+            eta = ETA(end - start)
+
+        for read in bam.fetch(working_chrom, start, end):
+            if not quiet and eta:
+                if callback:
+                    eta.print_status(read.pos - start, extra=callback(read))
+                else:
+                    eta.print_status(read.pos - start, extra='%s:%s %s' % (bam.getrname(read.tid), read.pos, read.qname))
+
+            yield read
 
     if eta:
         eta.done()
 
 
+def bam_batch_reads(bam, quiet=False):
+    '''
+    Batch mapping for the same reads (qname) together, this way
+    they can all be compared/converted together.
+    '''
+    reads = []
+    last = None
+    for read in bam_iter(bam, quiet=quiet):
+        if last and read.qname != last:
+            yield reads
+            reads = []
+        last = read.qname
+        reads.append(read)
+
+    if reads:
+        yield reads
+
+
 bam_cigar = ['M', 'I', 'D', 'N', 'S', 'H', 'P', '=', 'X']
 bam_cigar_op = {
     'M': 0,
@@ -105,7 +156,9 @@ def cigar_tostr(cigar):
     >>> cigar_tostr(((0, 1), (1, 1), (2, 1), (3, 1), (4, 1), (5, 1), (6, 1)))
     '1M1I1D1N1S1H1P'
     '''
+    
     s = ''
+
     for op, size in cigar:
         s += '%s%s' % (size, bam_cigar[op])
 
@@ -507,6 +560,12 @@ def region_pos_to_genomic_pos(name, start, cigar):
     >>> region_pos_to_genomic_pos('chr1:1000-1050,2000-2050', 25, [(5, 25), (0, 75)])
     ('chr1', 1025, [(5, 25), (0, 25), (3, 950), (0, 50)])
 
+    >>> region_pos_to_genomic_pos('chr1:1000-1050,2000-2050', 25, [(5, 25), (0, 75)])
+    ('chr1', 1025, [(5, 25), (0, 25), (3, 950), (0, 50)])
+
+    >>> region_pos_to_genomic_pos('chr7:16829153-16829246,16829246-16829339', 62, cigar_fromstr('83M18S'))
+    ('chr7', 16829215, [(0, 31), (3, 0), (0, 52), (4, 18)])
+
 
     '''
 
@@ -709,6 +768,8 @@ def read_cigar_at_pos(cigar, qpos, is_del):
             pass
         elif op == 4:
             pos += length
+        elif op == 5:
+            pass
         else:
             raise ValueError("Unsupported CIGAR operation: %s" % op)
 
@@ -720,39 +781,96 @@ def read_cigar_at_pos(cigar, qpos, is_del):
     return None
 
 
+def cleancigar(cigar):
+    '''
+    Cleans a CIGAR alignment to remove zero length operations
+
+    >>> cigar_tostr(cleancigar(cigar_fromstr('31M0N52M18S')))
+    '83M18S'
+
+    '''
+    newcigar = []
+
+    changed = False
+    zero = False
+
+    for op, size in cigar:
+        if size > 0:
+            if zero and newcigar:
+                if newcigar[-1][0] == op:
+                    newsize = newcigar[-1][1] + size
+                    newcigar[-1] = (op, newsize)
+                zero = 0
+            else:
+                newcigar.append((op, size))
+        else:
+            changed = True
+            zero = True
+
+    if changed:
+        return newcigar
+
+    return None
+
+
 def read_cleancigar(read):
     '''
-    Cleans the CIGAR string for a read to remove any operations that are zero length.
+    Replaces the CIGAR string for a read to remove any operations that are zero length.
+
+    This happens to the read in-place
     '''
     if read.is_unmapped:
         return False
 
     newcigar = []
-    changed = False
-    last_op = None
-    size_acc = 0
 
-    for op, size in read.cigar:
-        if size > 0:
-            if last_op == op:
-                size_acc += size
-            else:
-                if last_op:
-                    newcigar.append((last_op, size_acc))
-                last_op = op
-                size_acc = size
-        else:
-            changed = True
+    newcigar = cleancigar(read.cigar)
 
-    newcigar.append((last_op, size_acc))
-
-    if changed:
+    if newcigar:
         read.cigar = newcigar
         return True
 
     return False
 
 
+def read_to_unmapped(read, ref=None):
+    '''
+    Converts a read from mapped to unmapped.
+
+    Sets the 'ZR' tag to indicate the original ref/pos/cigar (if ref is passed)
+    '''
+
+    newread = pysam.AlignedRead()
+
+    if ref:
+        tags = [('ZR', '%s:%s:%s' % (ref, read.pos, cigar_tostr(read.cigar)))]
+
+    newread.is_unmapped = True
+    newread.mapq = 0
+    newread.tlen = 0
+    newread.pos = -1
+    newread.pnext = -1
+    newread.rnext = -1
+    newread.tid = -1
+
+    newread.qname = read.qname
+
+    if read.is_paired:
+        newread.is_paired = True
+
+    if not read.is_unmapped and read.is_reverse:
+        newread.seq = ngsutils.support.revcomp(read.seq)
+        newread.qual = read.qual[::-1]
+    else:        
+        newread.seq = read.seq
+        newread.qual = read.qual
+
+    newread.tags = tags
+
+    return newread
+
+
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
