@@ -72,7 +72,8 @@ Currently, the available filters are:
                                Note: If this is a large dataset, use
                                "bamutils extract" instead.
 
-    -excluderef refname        Exclude reads mapping to a particular reference
+    -includeref refname        Exclude reads NOT mapped to a reference
+    -excluderef refname        Exclude reads mapped to a particular reference
                                (e.g. chrM, or _dup chromosomes)
 
     -whitelist fname           Remove reads that aren't on this list (by name)
@@ -187,9 +188,9 @@ class UniqueStart(object):
             start_pos = read.aend
 
             # clean up hash if necesary
-            # Allow up to 10k over, to balance cleaning up the rev_pos hash
+            # Allow up to 100k over, to balance cleaning up the rev_pos hash
             # and memory
-            if read.pos > (self.last_rev_pos + 10000):
+            if read.pos > (self.last_rev_pos + 100000):
                 self.last_rev_pos = start_pos
                 del_list = []
                 for k in self.rev_pos:
@@ -201,7 +202,6 @@ class UniqueStart(object):
 
             if not start_pos in self.rev_pos:
                 self.rev_pos.add(start_pos)
-
                 return True
             return False
         else:
@@ -337,6 +337,22 @@ class ExcludeRef(object):
     def close(self):
         pass
 
+class IncludeRef(object):
+    def __init__(self, ref):
+        self.ref = ref
+
+    def filter(self, bam, read):
+        if not read.is_unmapped:
+            if bam.getrname(read.tid) == self.ref:
+                    return True
+        return False
+
+    def __repr__(self):
+        return 'Including: %s' % (self.ref)
+
+    def close(self):
+        pass
+
 
 class ExcludeBED(object):
     def __init__(self, fname, nostrand=None):
@@ -751,6 +767,7 @@ _criteria = {
     'exclude': ExcludeRegion,
     'excludebed': ExcludeBED,
     'excluderef': ExcludeRef,
+    'includeref': IncludeRef,
     'include': IncludeRegion,
     'includebed': IncludeBED,
     'whitelist': Whitelist,
