@@ -29,9 +29,11 @@ except:
     import pickle
 
 
-class GTF(object):
-    _version = 1.1
 
+class GTF(object):
+    _version = 1.3
+    __binsize = 10000
+    
     def __init__(self, filename=None, cache_enabled=True, quiet=False, fileobj=None):
         if not filename and not fileobj:
             raise ValueError('Must pass either a filename or a fileobj')
@@ -47,7 +49,7 @@ class GTF(object):
 
         self._genes = {}
         self._pos = 0
-        self._gene_order = {}
+        self._gene_bins = {}
         self._gene_names = {}
         self._gene_ids = {}
         warned = False
@@ -57,7 +59,8 @@ class GTF(object):
 
         if not self._genes:
             if not quiet:
-                sys.stderr.write('Reading GTF file...\n')
+                sys.stderr.write('Reading GTF file... (%s) \n' % filename)
+
             for line in fobj:
                 try:
                     idx = line.find('#')
@@ -82,13 +85,13 @@ class GTF(object):
                     elif 'gene_name' in attributes:  # use gene_name if we have it.
                         gid = attributes['gene_name']
 
-                    elif 'tss_id' in attributes:  # iGenomes GTF files... are strange. use gene_name first.
-                        gid = attributes['tss_id']
+                    # elif 'tss_id' in attributes:  # iGenomes GTF files... are strange. use gene_name first.
+                    #     gid = attributes['tss_id']
 
                     else:
                         gid = attributes['gene_id']
                         if not warned and not quiet:
-                            sys.stderr.write('\nGTF file missing isoform annotation! Each transcript will be treated separately. (%s)\n' % gid)
+                            sys.stderr.write('\nGTF file potentially missing isoform annotation! Each transcript may be treated separately. (%s)\n' % gid)
                             sys.stderr.write('%s\n\n' % (str(attributes)))
                             warned = True
                     if eta:
@@ -121,12 +124,15 @@ class GTF(object):
 
             for gid in self._genes:
                 gene = self._genes[gid]
-                if not gene.chrom in self._gene_order:
-                    self._gene_order[gene.chrom] = []
-                self._gene_order[gene.chrom].append((gene.start, gid))
 
-            for chrom in self._gene_order:
-                self._gene_order[chrom].sort()
+                start_bin = gene.start / GTF.__binsize
+                end_bin = gene.end / GTF.__binsize
+
+                for bin in xrange(start_bin, end_bin+1):
+                    if not (gene.chrom, bin) in self._gene_bins:
+                        self._gene_bins[(gene.chrom, bin)] = [gid]
+                    else:
+                        self._gene_bins[(gene.chrom, bin)].append(gid)
 
             if cache_enabled:
                 try:
@@ -140,9 +146,9 @@ class GTF(object):
         started_t = datetime.datetime.now()
         try:
             with open(cachefile) as cache:
-                version, genes, gene_order, gene_names, gene_ids = pickle.load(cache)
+                version, genes, gene_bins, gene_names, gene_ids = pickle.load(cache)
                 if version == GTF._version:
-                    self._genes, self._gene_order = genes, gene_order
+                    self._genes, self._gene_bins = genes, gene_bins
                     self._gene_names = gene_names
                     self._gene_ids = gene_ids
                     sys.stderr.write('(%s sec)\n' % (datetime.datetime.now() - started_t).seconds)
@@ -150,13 +156,13 @@ class GTF(object):
                     sys.stderr.write('Error reading cached file... Processing original file.\n')
         except:
             self._genes = {}
-            self._gene_order = {}
+            self._gene_bins = {}
             sys.stderr.write('Failed reading cache! Processing original file.\n')
 
     def _write_cache(self, cachefile):
         sys.stderr.write('(saving GTF cache)...')
         with open(cachefile, 'w') as cache:
-            pickle.dump((GTF._version, self._genes, self._gene_order, self._gene_names, self._gene_ids), cache)
+            pickle.dump((GTF._version, self._genes, self._gene_bins, self._gene_names, self._gene_ids), cache)
         sys.stderr.write('\n')
 
     def fsize(self):
@@ -166,30 +172,40 @@ class GTF(object):
         return self._pos
 
     def find(self, chrom, start, end=None, strand=None):
-        if chrom not in self._gene_order:
-            return
-
         if not end:
             end = start
 
         if end < start:
             raise ValueError('[gtf.find] Error: End must be smaller than start!')
 
-        for g_start, gid in self._gene_order[chrom]:
-            g_start = self._genes[gid].start
-            g_end = self._genes[gid].end
-            g_strand = self._genes[gid].strand
+        startbin = start / GTF.__binsize
+        if end:
+            endbin = end / GTF.__binsize
+        else:
+            endbin = startbin
 
-            if strand and g_strand != strand:
-                continue
+        proc_list = set()
+        for bin in xrange(startbin, endbin + 1):
+            if (chrom, bin) in self._gene_bins:
+                for gid in self._gene_bins[(chrom, bin)]:
+                    if gid in proc_list:
+                        continue
 
-            if g_start <= start <= g_end or g_start <= end <= g_end:
-                # gene spans the query boundary
-                yield self._genes[gid]
-            elif start <= g_start <= end and start <= g_end <= end:
-                # gene is completely inside boundary
-                yield self._genes[gid]
-        return
+                    proc_list.add(gid)
+
+                    g_start = self._genes[gid].start
+                    g_end = self._genes[gid].end
+                    g_strand = self._genes[gid].strand
+
+                    if strand and g_strand != strand:
+                        continue
+
+                    if g_start <= start <= g_end or g_start <= end <= g_end:
+                        # gene spans the query boundary
+                        yield self._genes[gid]
+                    elif start <= g_start <= end and start <= g_end <= end:
+                        # gene is completely inside boundary
+                        yield self._genes[gid]
 
     def get_by_id(self, gene_id):
         if gene_id in self._gene_ids:
@@ -207,9 +223,14 @@ class GTF(object):
     @property
     def genes(self):
         self._pos = 0
-        for chrom in sorted(self._gene_order):
-            for start, gene_id in self._gene_order[chrom]:
+        proc_list = set()
+        for chrbin in sorted(self._gene_bins):
+            for gene_id in self._gene_bins[chrbin]:
+                if gene_id in proc_list:
+                    continue
+
                 yield self._genes[gene_id]
+                proc_list.add(gene_id)
                 self._pos += 1
 
 
@@ -218,19 +239,20 @@ class _GTFGene(object):
     Stores info for a single gene_id
 
     A gene consists of one or more transcripts. It *must* have a gene_id, and chrom
-    It may also contain a gene_name and an isoform_id.
+    It may also contain a other attributes (isoform_id, gene_biotype, etc).
 
-    The gid is the 'gene_id' from the GTF file *unless* there is an isoform_id
-    attribute. If 'isoform_id' is present, this is used for the gid.
+    The gid is used to merge multiple transcripts together into one GTFGene record.
+    This is a separate attribute that will be either (in order): 
 
+    isoform_id, gene_name, gene_id
 
     """
 
-    def __init__(self, gid, chrom, source, gene_id, gene_name=None, isoform_id=None, **extras):
+    def __init__(self, gid, chrom, source, gene_id, gene_name=None, **attributes):
         self.gid = gid
         self.gene_id = gene_id
         self.gene_name = gene_name if gene_name else gene_id
-        self.isoform_id = isoform_id if isoform_id else gene_id
+        self.attributes = attributes
 
         self.chrom = chrom
         self.source = source
@@ -337,7 +359,7 @@ class _GTFTranscript(object):
         if self._cds:
             return self._cds
         else:
-            return [(self.start, self.end)]
+            return []
 
     @property
     def has_cds(self):
@@ -347,21 +369,49 @@ class _GTFTranscript(object):
 
     @property
     def utr_5(self):
+        utr = []
         if self._cds and self._exons:
             if self.strand == '+':
-                return (self._exons[0][0], self._cds[0][0])
+                cds_start = self._cds[0][0]
+                for s, e in self._exons:
+                    if e < cds_start:
+                        utr.append((s, e))
+                    else:
+                        utr.append((s, cds_start))
+                        break
             else:
-                return (self._exons[-1][1], self._cds[-1][1])
-        return None
+                cds_start = self._cds[-1][1]
+                for s, e in self._exons[::-1]:
+                    if s > cds_start:
+                        utr.append((s, e))
+                    else:
+                        utr.append((cds_start, e))
+                        break
+                utr.sort()
+        return utr
 
     @property
     def utr_3(self):
+        utr = []
         if self._cds and self._exons:
             if self.strand == '+':
-                return (self._cds[-1][1], self._exons[-1][1])
+                cds_end = self._cds[-1][1]
+                for s, e in self._exons[::-1]:
+                    if s > cds_end:
+                        utr.append((s, e))
+                    else:
+                        utr.append((cds_end, e))
+                        break
+                utr.sort()
             else:
-                return (self._exons[-1][0], self._cds[-1][0])
-        return None
+                cds_end = self._cds[0][0]
+                for s, e in self._exons:
+                    if e < cds_end:
+                        utr.append((s, e))
+                    else:
+                        utr.append((s, cds_end))
+                        break
+        return utr
 
     @property
     def start_codon(self):
